/*******************************************************************************
  Timer Static Driver File

  File Name:
    drv_tmr_static.c

  Company:
    Microchip Technology Inc.   

  Summary:
    Timer driver implementation for the static single instance driver.

  Description:
    The Timer device driver provides a simple interface to manage the Timer
    modules on Microchip microcontrollers.
    
  Remarks:
    None
 *******************************************************************************/

/*******************************************************************************
Copyright (c) 2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublTMRense terms in the accompanying lTMRense agreement).

You should refer to the lTMRense agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTTMRULAR PURPOSE.
IN NO EVENT SHALL MTMRROCHIP OR ITS LTMRENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRTMRT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVTMRES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Header Includes
// *****************************************************************************
// *****************************************************************************
#include "driver/tmr/drv_tmr_static.h"
#include "driver/tmr/src/drv_tmr_variant_mapping.h"

typedef struct
{
    DRV_TMR_CALLBACK alarmFunc;  // For alarm registering
    uint32_t alarmCount;    // For AlarmHasElapsed function
    bool    alarmEnabled;   // For Enable/Disable function
    bool    alarmPeriodic;      // Keep Alarm enabled or disable it
    uintptr_t   alarmContext;   // For Alarm Callback
    uint32_t    alarmPeriod;    // For Period Set/Get
} DRV_TMR_ALARM_OBJ;

static bool _DRV_TMR_ClockSourceSet(TMR_MODULE_ID timerId, DRV_TMR_CLK_SOURCES clockSource)
{
    bool clockSet = true;
    /* Clock Source Selection */
    if(clockSource == DRV_TMR_CLKSOURCE_INTERNAL)
    {
        if ( PLIB_TMR_ExistsClockSource ( timerId ) )
        {               
            PLIB_TMR_ClockSourceSelect ( timerId, TMR_CLOCK_SOURCE_PERIPHERAL_CLOCK );           
        }
        else
        {
            /* If clock source feature doesn't exist for any specific timer module instance,
            then by default internal peripheral clock is considered as timer source, so do nothing */ 
        }
    }
    /* External Synchronous Clock Source Selection */
    else if(!(clockSource & 0x10))
    {
        if ( PLIB_TMR_ExistsClockSource ( timerId ) )
        {               
            if ( PLIB_TMR_ExistsClockSourceSync ( timerId )  )
            {
                PLIB_TMR_ClockSourceSelect ( timerId, (TMR_CLOCK_SOURCE)(clockSource & 0x0F) );                
                PLIB_TMR_ClockSourceExternalSyncEnable ( timerId );                    
            }
            /* If Synchronization feature doesn't exist for any specific timer module 
            instance with external clock source then it is synchronous by default */
            else if (clockSource == DRV_TMR_CLKSOURCE_EXTERNAL_SYNCHRONOUS)
            {
                PLIB_TMR_ClockSourceSelect ( timerId, TMR_CLOCK_SOURCE_EXTERNAL_INPUT_PIN );
            }
            else
            {
                clockSet = false;
            }  
        }
        else
        {
            clockSet = false;
        }        
    }
    /* External Asynchronous Clock Source Selection */
    else if(clockSource & 0x10)
    {
        if ( PLIB_TMR_ExistsClockSourceSync ( timerId ) )
        {
            PLIB_TMR_ClockSourceSelect ( timerId, (TMR_CLOCK_SOURCE)(clockSource & 0x0F) );
            PLIB_TMR_ClockSourceExternalSyncDisable ( timerId );
        }
        else
        {
            clockSet = false;
        }        
    }
    
    return clockSet;
}

// Prescaler selection
static bool _DRV_TMR_ClockPrescaleSet(TMR_MODULE_ID timerId, TMR_PRESCALE  prescale)
{
    if( PLIB_TMR_ExistsPrescale( timerId ) )
    {
        PLIB_TMR_PrescaleSelect( timerId , prescale );
        return true;
    }
    return false;
}


// *****************************************************************************
// *****************************************************************************
// Section: Instance 1 static driver data
// *****************************************************************************
// *****************************************************************************

static bool                   DRV_TMR1_Running;

// *****************************************************************************
// *****************************************************************************
// Section: Instance 1 static driver functions
// *****************************************************************************
// *****************************************************************************
void DRV_TMR1_Initialize(void)
{   
    /* Initialize Timer Instance1 */
    /* Disable Timer */
    PLIB_TMR_Stop(TMR_ID_4);
    /* Select clock source */
    PLIB_TMR_ClockSourceSelect ( TMR_ID_4, TMR_CLOCK_SOURCE_PERIPHERAL_CLOCK );
    /* Select prescalar value */
    PLIB_TMR_PrescaleSelect(TMR_ID_4, TMR_PRESCALE_VALUE_64);
    /* Enable 16 bit mode */
    PLIB_TMR_Mode16BitEnable(TMR_ID_4);
    /* Clear counter */ 
    PLIB_TMR_Counter16BitClear(TMR_ID_4);
    /*Set period */ 
    PLIB_TMR_Period16BitSet(TMR_ID_4, 31250);
    /* Setup Interrupt */   
    PLIB_INT_VectorPrioritySet(INT_ID_0, INT_VECTOR_T4, INT_PRIORITY_LEVEL1);
    PLIB_INT_VectorSubPrioritySet(INT_ID_0, INT_VECTOR_T4, INT_SUBPRIORITY_LEVEL0);          
}

static void _DRV_TMR1_Resume(bool resume)
{
    if (resume)
    {
        PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_4);
        PLIB_INT_SourceEnable(INT_ID_0, INT_SOURCE_TIMER_4);
        PLIB_TMR_Start(TMR_ID_4);
    }
}

bool DRV_TMR1_Start(void)
{
    /* Start Timer*/
    _DRV_TMR1_Resume(true);
    DRV_TMR1_Running = true;
    
    return true;
}

static bool _DRV_TMR1_Suspend(void)
{
    if (DRV_TMR1_Running)
    {
        PLIB_INT_SourceDisable(INT_ID_0, INT_SOURCE_TIMER_4);
        PLIB_TMR_Stop(TMR_ID_4);
        return (true);
    }
    
    return (false);
}

void DRV_TMR1_Stop(void)
{
    _DRV_TMR1_Suspend();
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_4);
    DRV_TMR1_Running = false;
}

DRV_TMR_CLIENT_STATUS DRV_TMR1_ClientStatus ( void )
{
    if (DRV_TMR1_Running)
        return DRV_TMR_CLIENT_STATUS_RUNNING;
    else
        return DRV_TMR_CLIENT_STATUS_READY;
}

void DRV_TMR1_CounterValueSet(uint32_t value)
{
    /* Set 16-bit counter value*/
    PLIB_TMR_Counter16BitSet(TMR_ID_4, (uint16_t)value);
}

uint32_t DRV_TMR1_CounterValueGet(void)
{
    /* Get 16-bit counter value*/
    return (uint32_t) PLIB_TMR_Counter16BitGet(TMR_ID_4);
}

void DRV_TMR1_CounterClear(void)
{
    /* Clear 16-bit counter value*/
    PLIB_TMR_Counter16BitClear(TMR_ID_4);
}

DRV_TMR_OPERATION_MODE DRV_TMR1_DividerRangeGet
(
	DRV_TMR_DIVIDER_RANGE * pDivRange
)
{
	if(pDivRange)
	{
        pDivRange->dividerMax = DRV_TIMER_DIVIDER_MAX_16BIT;
        pDivRange->dividerMin = DRV_TIMER_DIVIDER_MIN_16BIT;
		pDivRange->dividerStep = 1;
		return DRV_TMR_OPERATION_MODE_16_BIT;
	}
	return DRV_TMR_OPERATION_MODE_NONE;
}

uint32_t DRV_TMR1_CounterFrequencyGet(void)
{
    uint32_t prescale, tmrBaseFreq;
    
    tmrBaseFreq = SYS_CLK_PeripheralFrequencyGet ( CLK_BUS_FOR_TIMER_PERIPHERAL );
    prescale = PLIB_TMR_PrescaleGet(TMR_ID_4);
    return ( tmrBaseFreq / prescale );
}

TMR_PRESCALE DRV_TMR1_PrescalerGet(void)
{
    uint16_t prescale_value;
    /* Call the PLIB directly */
    prescale_value = PLIB_TMR_PrescaleGet(TMR_ID_4);
    
    switch(prescale_value)
    {
        case 1: return TMR_PRESCALE_VALUE_1;
        case 2: return TMR_PRESCALE_VALUE_2;
        case 4: return TMR_PRESCALE_VALUE_4;
        case 8: return TMR_PRESCALE_VALUE_8;
        case 16: return TMR_PRESCALE_VALUE_16;
        case 32: return TMR_PRESCALE_VALUE_32;
        case 64: return TMR_PRESCALE_VALUE_64;
        case 256: return TMR_PRESCALE_VALUE_256;
        default: return TMR_PRESCALE_VALUE_1;
    }
}

void DRV_TMR1_PeriodValueSet(uint32_t value)
{
    /* Set 16-bit counter value*/
    PLIB_TMR_Period16BitSet(TMR_ID_4, (uint16_t)value);
}

uint32_t DRV_TMR1_PeriodValueGet(void)
{
    /* Get 16-bit counter value*/
    return (uint32_t) PLIB_TMR_Period16BitGet(TMR_ID_4);
}

void DRV_TMR1_StopInIdleDisable(void)
{
    PLIB_TMR_StopInIdleDisable(TMR_ID_4);
}

void DRV_TMR1_StopInIdleEnable(void)
{
    PLIB_TMR_StopInIdleDisable(TMR_ID_4);
}

bool DRV_TMR1_ClockSet
(
    DRV_TMR_CLK_SOURCES clockSource,
    TMR_PRESCALE        preScale
)
{
    bool success = false;
    bool resume = _DRV_TMR1_Suspend();
    
    if (_DRV_TMR_ClockSourceSet(TMR_ID_4, clockSource) &&
        _DRV_TMR_ClockPrescaleSet(TMR_ID_4, preScale))
    {
        success = true;
    }
    
    _DRV_TMR1_Resume(resume);
    return success;
}

// *****************************************************************************
// *****************************************************************************
// Section: Instance 2 static driver data
// *****************************************************************************
// *****************************************************************************

static bool                   DRV_TMR2_Running;

// *****************************************************************************
// *****************************************************************************
// Section: Instance 2 static driver functions
// *****************************************************************************
// *****************************************************************************
void DRV_TMR2_Initialize(void)
{   
    /* Initialize Timer Instance2 */
    /* Disable Timer */
    PLIB_TMR_Stop(TMR_ID_5);
    /* Select clock source */
    PLIB_TMR_ClockSourceSelect ( TMR_ID_5, TMR_CLOCK_SOURCE_PERIPHERAL_CLOCK );
    /* Select prescalar value */
    PLIB_TMR_PrescaleSelect(TMR_ID_5, TMR_PRESCALE_VALUE_8);
    /* Enable 16 bit mode */
    PLIB_TMR_Mode16BitEnable(TMR_ID_5);
    /* Clear counter */ 
    PLIB_TMR_Counter16BitClear(TMR_ID_5);
    /*Set period */ 
    PLIB_TMR_Period16BitSet(TMR_ID_5, 625);
    /* Setup Interrupt */   
    PLIB_INT_VectorPrioritySet(INT_ID_0, INT_VECTOR_T5, INT_PRIORITY_LEVEL1);
    PLIB_INT_VectorSubPrioritySet(INT_ID_0, INT_VECTOR_T5, INT_SUBPRIORITY_LEVEL0);          
}

static void _DRV_TMR2_Resume(bool resume)
{
    if (resume)
    {
        PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_5);
        PLIB_INT_SourceEnable(INT_ID_0, INT_SOURCE_TIMER_5);
        PLIB_TMR_Start(TMR_ID_5);
    }
}

bool DRV_TMR2_Start(void)
{
    /* Start Timer*/
    _DRV_TMR2_Resume(true);
    DRV_TMR2_Running = true;
    
    return true;
}

static bool _DRV_TMR2_Suspend(void)
{
    if (DRV_TMR2_Running)
    {
        PLIB_INT_SourceDisable(INT_ID_0, INT_SOURCE_TIMER_5);
        PLIB_TMR_Stop(TMR_ID_5);
        return (true);
    }
    
    return (false);
}

void DRV_TMR2_Stop(void)
{
    _DRV_TMR2_Suspend();
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_5);
    DRV_TMR2_Running = false;
}

DRV_TMR_CLIENT_STATUS DRV_TMR2_ClientStatus ( void )
{
    if (DRV_TMR2_Running)
        return DRV_TMR_CLIENT_STATUS_RUNNING;
    else
        return DRV_TMR_CLIENT_STATUS_READY;
}

void DRV_TMR2_CounterValueSet(uint32_t value)
{
    /* Set 16-bit counter value*/
    PLIB_TMR_Counter16BitSet(TMR_ID_5, (uint16_t)value);
}

uint32_t DRV_TMR2_CounterValueGet(void)
{
    /* Get 16-bit counter value*/
    return (uint32_t) PLIB_TMR_Counter16BitGet(TMR_ID_5);
}

void DRV_TMR2_CounterClear(void)
{
    /* Clear 16-bit counter value*/
    PLIB_TMR_Counter16BitClear(TMR_ID_5);
}

DRV_TMR_OPERATION_MODE DRV_TMR2_DividerRangeGet
(
	DRV_TMR_DIVIDER_RANGE * pDivRange
)
{
	if(pDivRange)
	{
        pDivRange->dividerMax = DRV_TIMER_DIVIDER_MAX_16BIT;
        pDivRange->dividerMin = DRV_TIMER_DIVIDER_MIN_16BIT;
		pDivRange->dividerStep = 1;
		return DRV_TMR_OPERATION_MODE_16_BIT;
	}
	return DRV_TMR_OPERATION_MODE_NONE;
}

uint32_t DRV_TMR2_CounterFrequencyGet(void)
{
    uint32_t prescale, tmrBaseFreq;
    
    tmrBaseFreq = SYS_CLK_PeripheralFrequencyGet ( CLK_BUS_FOR_TIMER_PERIPHERAL );
    prescale = PLIB_TMR_PrescaleGet(TMR_ID_5);
    return ( tmrBaseFreq / prescale );
}

TMR_PRESCALE DRV_TMR2_PrescalerGet(void)
{
    uint16_t prescale_value;
    /* Call the PLIB directly */
    prescale_value = PLIB_TMR_PrescaleGet(TMR_ID_5);
    
    switch(prescale_value)
    {
        case 1: return TMR_PRESCALE_VALUE_1;
        case 2: return TMR_PRESCALE_VALUE_2;
        case 4: return TMR_PRESCALE_VALUE_4;
        case 8: return TMR_PRESCALE_VALUE_8;
        case 16: return TMR_PRESCALE_VALUE_16;
        case 32: return TMR_PRESCALE_VALUE_32;
        case 64: return TMR_PRESCALE_VALUE_64;
        case 256: return TMR_PRESCALE_VALUE_256;
        default: return TMR_PRESCALE_VALUE_1;
    }
}

void DRV_TMR2_PeriodValueSet(uint32_t value)
{
    /* Set 16-bit counter value*/
    PLIB_TMR_Period16BitSet(TMR_ID_5, (uint16_t)value);
}

uint32_t DRV_TMR2_PeriodValueGet(void)
{
    /* Get 16-bit counter value*/
    return (uint32_t) PLIB_TMR_Period16BitGet(TMR_ID_5);
}

void DRV_TMR2_StopInIdleDisable(void)
{
    PLIB_TMR_StopInIdleDisable(TMR_ID_5);
}

void DRV_TMR2_StopInIdleEnable(void)
{
    PLIB_TMR_StopInIdleDisable(TMR_ID_5);
}

bool DRV_TMR2_ClockSet
(
    DRV_TMR_CLK_SOURCES clockSource,
    TMR_PRESCALE        preScale
)
{
    bool success = false;
    bool resume = _DRV_TMR2_Suspend();
    
    if (_DRV_TMR_ClockSourceSet(TMR_ID_5, clockSource) &&
        _DRV_TMR_ClockPrescaleSet(TMR_ID_5, preScale))
    {
        success = true;
    }
    
    _DRV_TMR2_Resume(resume);
    return success;
}

// *****************************************************************************
// *****************************************************************************
// Section: Instance 3 static driver data
// *****************************************************************************
// *****************************************************************************

static DRV_TMR_ALARM_OBJ      DRV_TMR3_DATA;
static bool                   DRV_TMR3_Running;

// *****************************************************************************
// *****************************************************************************
// Section: Instance 3 static driver functions
// *****************************************************************************
// *****************************************************************************
void DRV_TMR3_Initialize(void)
{   
    /* Initialize Timer Instance3 */
    /* Disable Timer */
    PLIB_TMR_Stop(TMR_ID_3);
    /* Select clock source */
    PLIB_TMR_ClockSourceSelect ( TMR_ID_3, TMR_CLOCK_SOURCE_PERIPHERAL_CLOCK );
    /* Select prescalar value */
    PLIB_TMR_PrescaleSelect(TMR_ID_3, TMR_PRESCALE_VALUE_64);
    /* Enable 16 bit mode */
    PLIB_TMR_Mode16BitEnable(TMR_ID_3);
    /* Clear counter */ 
    PLIB_TMR_Counter16BitClear(TMR_ID_3);
    /*Set period */ 
    PLIB_TMR_Period16BitSet(TMR_ID_3, 31250);
    /* Initialize the data structure */
    DRV_TMR3_DATA.alarmFunc = NULL;
    DRV_TMR3_DATA.alarmCount = 0;
    DRV_TMR3_DATA.alarmEnabled = false;
    DRV_TMR3_DATA.alarmPeriodic = false;
    /* Setup Interrupt */   
    PLIB_INT_VectorPrioritySet(INT_ID_0, INT_VECTOR_T3, INT_PRIORITY_LEVEL1);
    PLIB_INT_VectorSubPrioritySet(INT_ID_0, INT_VECTOR_T3, INT_SUBPRIORITY_LEVEL0);          
}

static void _DRV_TMR3_Resume(bool resume)
{
    if (resume)
    {
        PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_3);
        PLIB_INT_SourceEnable(INT_ID_0, INT_SOURCE_TIMER_3);
        PLIB_TMR_Start(TMR_ID_3);
    }
}

bool DRV_TMR3_Start(void)
{
    /* Start Timer*/
    _DRV_TMR3_Resume(true);
    DRV_TMR3_Running = true;
    
    return true;
}

static bool _DRV_TMR3_Suspend(void)
{
    if (DRV_TMR3_Running)
    {
        PLIB_INT_SourceDisable(INT_ID_0, INT_SOURCE_TIMER_3);
        PLIB_TMR_Stop(TMR_ID_3);
        return (true);
    }
    
    return (false);
}

void DRV_TMR3_Stop(void)
{
    _DRV_TMR3_Suspend();
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_3);
    DRV_TMR3_Running = false;
}

DRV_TMR_CLIENT_STATUS DRV_TMR3_ClientStatus ( void )
{
    if (DRV_TMR3_Running)
        return DRV_TMR_CLIENT_STATUS_RUNNING;
    else
        return DRV_TMR_CLIENT_STATUS_READY;
}

void DRV_TMR3_CounterValueSet(uint32_t value)
{
    /* Set 16-bit counter value*/
    PLIB_TMR_Counter16BitSet(TMR_ID_3, (uint16_t)value);
}

uint32_t DRV_TMR3_CounterValueGet(void)
{
    /* Get 16-bit counter value*/
    return (uint32_t) PLIB_TMR_Counter16BitGet(TMR_ID_3);
}

void DRV_TMR3_CounterClear(void)
{
    /* Clear 16-bit counter value*/
    PLIB_TMR_Counter16BitClear(TMR_ID_3);
}

DRV_TMR_OPERATION_MODE DRV_TMR3_DividerRangeGet
(
	DRV_TMR_DIVIDER_RANGE * pDivRange
)
{
	if(pDivRange)
	{
        pDivRange->dividerMax = DRV_TIMER_DIVIDER_MAX_16BIT;
        pDivRange->dividerMin = DRV_TIMER_DIVIDER_MIN_16BIT;
		pDivRange->dividerStep = 1;
		return DRV_TMR_OPERATION_MODE_16_BIT;
	}
	return DRV_TMR_OPERATION_MODE_NONE;
}

uint32_t DRV_TMR3_CounterFrequencyGet(void)
{
    uint32_t prescale, tmrBaseFreq;
    
    tmrBaseFreq = SYS_CLK_PeripheralFrequencyGet ( CLK_BUS_FOR_TIMER_PERIPHERAL );
    prescale = PLIB_TMR_PrescaleGet(TMR_ID_3);
    return ( tmrBaseFreq / prescale );
}

TMR_PRESCALE DRV_TMR3_PrescalerGet(void)
{
    uint16_t prescale_value;
    /* Call the PLIB directly */
    prescale_value = PLIB_TMR_PrescaleGet(TMR_ID_3);
    
    switch(prescale_value)
    {
        case 1: return TMR_PRESCALE_VALUE_1;
        case 2: return TMR_PRESCALE_VALUE_2;
        case 4: return TMR_PRESCALE_VALUE_4;
        case 8: return TMR_PRESCALE_VALUE_8;
        case 16: return TMR_PRESCALE_VALUE_16;
        case 32: return TMR_PRESCALE_VALUE_32;
        case 64: return TMR_PRESCALE_VALUE_64;
        case 256: return TMR_PRESCALE_VALUE_256;
        default: return TMR_PRESCALE_VALUE_1;
    }
}

void DRV_TMR3_PeriodValueSet(uint32_t value)
{
    /* Set 16-bit counter value*/
    PLIB_TMR_Period16BitSet(TMR_ID_3, (uint16_t)value);
}

uint32_t DRV_TMR3_PeriodValueGet(void)
{
    /* Get 16-bit counter value*/
    return (uint32_t) PLIB_TMR_Period16BitGet(TMR_ID_3);
}

void DRV_TMR3_StopInIdleDisable(void)
{
    PLIB_TMR_StopInIdleDisable(TMR_ID_3);
}

void DRV_TMR3_StopInIdleEnable(void)
{
    PLIB_TMR_StopInIdleDisable(TMR_ID_3);
}

bool DRV_TMR3_ClockSet
(
    DRV_TMR_CLK_SOURCES clockSource,
    TMR_PRESCALE        preScale
)
{
    bool success = false;
    bool resume = _DRV_TMR3_Suspend();
    
    if (_DRV_TMR_ClockSourceSet(TMR_ID_3, clockSource) &&
        _DRV_TMR_ClockPrescaleSet(TMR_ID_3, preScale))
    {
        success = true;
    }
    
    _DRV_TMR3_Resume(resume);
    return success;
}

bool DRV_TMR3_AlarmRegister
(
    uint32_t divider, 
    bool isPeriodic, 
    uintptr_t context, 
    DRV_TMR_CALLBACK callBack 
)
{
    bool success = (divider >= DRV_TIMER_DIVIDER_MIN_16BIT && 
                    divider <= DRV_TIMER_DIVIDER_MAX_16BIT);
    if (success)
    {
        PLIB_INT_SourceDisable(INT_ID_0, INT_SOURCE_TIMER_3);
        PLIB_TMR_Stop(TMR_ID_3);
        PLIB_TMR_Period16BitSet(TMR_ID_3, (uint16_t)divider - 1);
        DRV_TMR3_DATA.alarmPeriodic = isPeriodic;
        DRV_TMR3_DATA.alarmContext = context;
        DRV_TMR3_DATA.alarmFunc = callBack;
        return (success);
    }
    else
        return false;
}

void DRV_TMR3_AlarmEnable(bool enable)
{
    if (DRV_TMR3_DATA.alarmFunc != NULL)
    {
        DRV_TMR3_DATA.alarmEnabled = enable;
        PLIB_INT_SourceEnable(INT_ID_0, INT_SOURCE_TIMER_3);
    }
}

bool DRV_TMR3_AlarmDisable(void)
{
    bool retVal = DRV_TMR3_DATA.alarmEnabled;
        PLIB_INT_SourceDisable(INT_ID_0, INT_SOURCE_TIMER_3);
    DRV_TMR3_DATA.alarmEnabled = false;
    
    return retVal;
}

void DRV_TMR3_AlarmPeriodSet(uint32_t value)
{
    bool resume = _DRV_TMR3_Suspend();
    DRV_TMR3_DATA.alarmPeriod = value;
    PLIB_TMR_Period16BitSet(TMR_ID_3, (uint16_t)value);
    _DRV_TMR3_Resume(resume);
}

uint32_t DRV_TMR3_AlarmPeriodGet(void)
{
    return PLIB_TMR_Period16BitGet(TMR_ID_3);
}

void DRV_TMR3_AlarmDeregister(void)
{
    DRV_TMR3_DATA.alarmEnabled = false;
    DRV_TMR3_DATA.alarmFunc = NULL;
}

uint32_t DRV_TMR3_AlarmHasElapsed(void)
{
    uint32_t alarmCountTemp = DRV_TMR3_DATA.alarmCount;
    
    DRV_TMR3_DATA.alarmCount = 0;
    return (alarmCountTemp);
}

void DRV_TMR3_Tasks(void)
{
    if (DRV_TMR3_DATA.alarmEnabled &&
        (DRV_TMR3_DATA.alarmFunc != NULL))
    {	
		DRV_TMR3_DATA.alarmCount++;
        DRV_TMR3_DATA.alarmFunc(
            DRV_TMR3_DATA.alarmContext, 
            DRV_TMR3_DATA.alarmCount);
        if (!DRV_TMR3_DATA.alarmPeriodic)
        {
            DRV_TMR3_AlarmDisable();
        }
    }
}
 
 
/*******************************************************************************
 End of File
*/
